@@ -26,6 +26,8 @@
 #include <unistd.h>
 #include <fcntl.h>
 
+#include <vicious.h>
+
 #include "gdm.h"
 #include "gdm-net.h"
 #include "server.h"
@@ -100,9 +102,9 @@ gdm_display_check_loop (GdmDisplay *disp)
 	  if (disp->type == TYPE_LOCAL ||
 	      disp->type == TYPE_FLEXI) {
 		  char *ls = g_strdup_printf
-			  (gdm_cons_i18n (N_("Failed to start the display server "
-					     "several times in a short time period; "
-					     "disabling display %s")), disp->name);
+			  (_("Failed to start the display server "
+			     "several times in a short time period; "
+			     "disabling display %s"), disp->name);
 		  gdm_text_message_dialog (ls);
 		  g_free (ls);
 	  }
@@ -150,7 +152,6 @@ gboolean
 gdm_display_manage (GdmDisplay *d)
 {
     pid_t pid;
-    int i;
     int fds[2];
 
     if (!d) 
@@ -170,7 +171,7 @@ gdm_display_manage (GdmDisplay *d)
     /* If we have an old slave process hanging around, kill it */
     if (d->slavepid) {
 	    if (kill (d->slavepid, SIGINT) == 0)
-		    waitpid (d->slavepid, 0, 0);
+		    ve_waitpid_no_signal (d->slavepid, 0, 0);
 	    d->slavepid = 0;
     }
 
@@ -195,18 +196,10 @@ gdm_display_manage (GdmDisplay *d)
 	gdm_connection_close (unixconn);
 	unixconn = NULL;
 
+	closelog ();
+
 	/* Close everything */
-	for (i = 0; i < sysconf (_SC_OPEN_MAX); i++) {
-		if (i != fds[0])
-			close(i);
-	}
-
-	d->slave_notify_conn = gdm_connection_open_fd (fds[0]);
-	gdm_connection_set_handler (d->slave_notify_conn,
-				    gdm_slave_handle_notify,
-				    d /* data */,
-				    NULL /* destroy_notify */);
-
+	gdm_close_all_descriptors (0 /* from */, fds[0] /* except */);
 
 	/* No error checking here - if it's messed the best response
          * is to ignore & try to continue */
@@ -214,6 +207,10 @@ gdm_display_manage (GdmDisplay *d)
 	open("/dev/null", O_RDWR); /* open stdout - fd 1 */
 	open("/dev/null", O_RDWR); /* open stderr - fd 2 */
 
+	openlog ("gdm", LOG_PID, LOG_DAEMON);
+
+	d->slave_notify_fd = fds[0];
+
 	if (d->type == TYPE_LOCAL) {
 	    gdm_slave_start (d);
 	    /* if we ever return, bad things are happening */
@@ -282,7 +279,7 @@ gdm_display_unmanage (GdmDisplay *d)
     /* Kill slave */
     if (d->slavepid != 0) {
 	if (kill (d->slavepid, SIGTERM) == 0)
-		waitpid (d->slavepid, 0, 0);
+		ve_waitpid_no_signal (d->slavepid, 0, 0);
 	d->slavepid = 0;
     }
     
@@ -312,10 +309,9 @@ gdm_display_dispose (GdmDisplay *d)
 	    gdm_connection_set_close_notify (conn, NULL, NULL);
     }
 
-    if (d->slave_notify_conn != NULL) {
-	    GdmConnection *conn = d->slave_notify_conn;
-	    d->slave_notify_conn = NULL;
-	    gdm_connection_close (conn);
+    if (d->slave_notify_fd >= 0) {
+	    close (d->slave_notify_fd);
+	    d->slave_notify_fd = -1;
     }
 
     if (d->master_notify_fd >= 0) {
