@@ -17,9 +17,13 @@
  */
 
 #include <config.h>
-#include <gnome.h>
+#include <libgnome/libgnome.h>
+#include <gtk/gtkmessagedialog.h>
 #include <syslog.h>
 #include <pwd.h>
+#include <grp.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 #ifdef HAVE_CRYPT
 #  include <crypt.h>
@@ -31,15 +35,12 @@
 #include "misc.h"
 #include "slave.h"
 #include "verify.h"
-
-static const gchar RCSid[]="$Id: verify-crypt.c,v 1.17 2001/09/27 04:10:06 jirka Exp $";
-
+#include "errorgui.h"
 
 /* Configuration option variables */
-extern gboolean GdmVerboseAuth;
 extern gboolean GdmAllowRoot;
 extern gboolean GdmAllowRemoteRoot;
-
+extern gint GdmRetryDelay;
 
 /**
  * gdm_verify_user:
@@ -67,14 +68,16 @@ gdm_verify_user (GdmDisplay *d,
 
     /* Ask for the user's login */
     if (username == NULL) {
+	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, _("Please enter your username"));
 	    login = gdm_slave_greeter_ctl (GDM_LOGIN, _("Username:"));
 	    if (login == NULL ||
-		gdm_slave_greeter_check_interruption (login)) {
+		gdm_slave_greeter_check_interruption ()) {
 		    if (local)
 			    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 		    g_free (login);
 		    return NULL;
 	    }
+	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, "");
     } else {
 	    login = g_strdup (username);
     }
@@ -92,7 +95,7 @@ gdm_verify_user (GdmDisplay *d,
 	    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password: "));
 	    if (passwd == NULL)
 		    passwd = g_strdup ("");
-	    if (gdm_slave_greeter_check_interruption (passwd)) {
+	    if (gdm_slave_greeter_check_interruption ()) {
 		    if (local)
 			    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 		    g_free (login);
@@ -106,18 +109,18 @@ gdm_verify_user (GdmDisplay *d,
 	    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 
     if (pwent == NULL) {
+	    sleep (GdmRetryDelay);
 	    gdm_error (_("Couldn't authenticate user"));
-	    if (GdmVerboseAuth) {
-		    auth_errmsg = g_strdup_printf
-			    (_("\nIncorrect username or password.  "
-			       "Letters must be typed in the correct case.  "  
-			       "Please be sure the Caps Lock key is not enabled"));
-		    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
-		    g_free (auth_errmsg);
-	    } else {
-		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
-						  _("Login incorrect"));
-	    }
+	    /* FIXME: Hmm, how are we sure that the login is username
+	     * and password.  That is the most common case but not
+	     * neccessairly true, this message needs to be changed
+	     * to allow for such cases */
+	    auth_errmsg = g_strdup_printf
+		    (_("\nIncorrect username or password.  "
+		       "Letters must be typed in the correct case.  "  
+		       "Please be sure the Caps Lock key is not enabled"));
+	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
+	    g_free (auth_errmsg);
 	    g_free (login);
 	    g_free (passwd);
 	    g_free (ppasswd);
@@ -127,17 +130,17 @@ gdm_verify_user (GdmDisplay *d,
     /* Check whether password is valid */
     if (ppasswd == NULL || (ppasswd[0] != '\0' &&
 			    strcmp (crypt (passwd, ppasswd), ppasswd) != 0)) {
-	    if (GdmVerboseAuth) {
-		    auth_errmsg = g_strdup_printf
-			    (_("\nIncorrect username or password.  "
-			       "Letters must be typed in the correct case.  "  
-			       "Please be sure the Caps Lock key is not enabled"));
-		    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
-		    g_free (auth_errmsg);
-	    } else {
-		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
-						  _("Login incorrect"));
-	    }
+	    sleep (GdmRetryDelay);
+	    /* FIXME: Hmm, how are we sure that the login is username
+	     * and password.  That is the most common case but not
+	     * neccessairly true, this message needs to be changed
+	     * to allow for such cases */
+	    auth_errmsg = g_strdup_printf
+		    (_("\nIncorrect username or password.  "
+		       "Letters must be typed in the correct case.  "  
+		       "Please be sure the Caps Lock key is not enabled"));
+	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
+	    g_free (auth_errmsg);
 	    g_free (login);
 	    g_free (passwd);
 	    g_free (ppasswd);
@@ -148,17 +151,12 @@ gdm_verify_user (GdmDisplay *d,
 	  ( ! GdmAllowRemoteRoot && ! local) ) &&
 	pwent->pw_uid == 0) {
 	    gdm_error (_("Root login disallowed on display '%s'"), display);
-  	    if (GdmVerboseAuth) {
- 		    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
-						  _("The system administrator "
-						    "is not allowed to login "
-						    "from this screen"));
- 		    /*gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
- 		      _("Root login disallowed"));*/
-  	    } else {
- 		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
-						  _("Login incorrect"));
-  	    }
+	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
+					  _("The system administrator "
+					    "is not allowed to login "
+					    "from this screen"));
+	    /*gdm_slave_greeter_ctl_no_ret (GDM_ERRDLG,
+	      _("Root login disallowed"));*/
 	    g_free (login);
 	    g_free (passwd);
 	    g_free (ppasswd);
@@ -171,17 +169,12 @@ gdm_verify_user (GdmDisplay *d,
 	 strcmp (pwent->pw_shell, "/bin/true") == 0 ||
 	 strcmp (pwent->pw_shell, "/bin/false") == 0)) {
 	    gdm_error (_("User %s not allowed to log in"), login);
-	    if (GdmVerboseAuth) {
-		    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
-						  _("\nThe system administrator"
-						    " has disabled your "
-						    "account."));
-		    /*gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
-		      _("Login disabled"));*/
-	    } else {
-		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
-						  _("Login incorrect"));
-	    }
+	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
+					  _("\nThe system administrator "
+					    "has disabled your "
+					    "account."));
+	    /*gdm_slave_greeter_ctl_no_ret (GDM_ERRDLG,
+	      _("Login disabled"));*/
 	    g_free (login);
 	    g_free (passwd);
 	    g_free (ppasswd);
@@ -190,6 +183,16 @@ gdm_verify_user (GdmDisplay *d,
 
     g_free (passwd);
     g_free (ppasswd);
+
+    if ( ! gdm_setup_gids (login, pwent->pw_gid)) {
+	    gdm_error (_("Cannot set user group for %s"), login);
+	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
+					  _("\nCannot set your user group, "
+					    "you will not be able to log in, "
+					    "please contact your system administrator."));
+	    g_free (login);
+	    return NULL;
+    }
     
     return login;
 }
@@ -203,10 +206,29 @@ gdm_verify_user (GdmDisplay *d,
  * session for this user
  */
 
-void
+gboolean
 gdm_verify_setup_user (GdmDisplay *d,
 		       const gchar *login, const gchar *display) 
 {
+	struct passwd *pwent;
+
+	pwent = getpwnam (login);
+	if (pwent == NULL) {
+		gdm_error (_("Cannot get passwd structure for %s"), login);
+		return FALSE;
+	}
+
+	if ( ! gdm_setup_gids (login, pwent->pw_gid)) {
+		gdm_error (_("Cannot set user group for %s"), login);
+		gdm_error_box (d,
+			       GTK_MESSAGE_ERROR,
+			       _("\nCannot set your user group, "
+				 "you will not be able to log in, "
+				 "please contact your system administrator."));
+		return FALSE;
+	}
+
+	return TRUE;
 }
 
 
@@ -219,6 +241,12 @@ gdm_verify_setup_user (GdmDisplay *d,
 void
 gdm_verify_cleanup (GdmDisplay *d)
 {
+	gid_t groups[1] = { 0 };
+
+	/* Clear the group setup */
+	setgid (0);
+	/* this will get rid of any suplementary groups etc... */
+	setgroups (1, groups);
 }
 
 /**
@@ -236,7 +264,7 @@ gdm_verify_check (void)
 
 /* used in pam */
 gboolean
-gdm_verify_open_session (GdmDisplay *d)
+gdm_verify_setup_env (GdmDisplay *d)
 {
 	return TRUE;
 }
